@@ -16,10 +16,6 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-## register GA for seriation
-
-
-
 #' Register a Genetic Algorithm Seriation Method
 #'
 #' Register a GA-based seriation metaheuristic for use with [seriate()].
@@ -27,24 +23,6 @@
 #' Registers the method \code{"GA"} for [seriate()]. This method can be used
 #' to optimize any criterion in package \pkg{seriation}.
 #'
-#' \code{control} for
-#' \code{seriate} with method \code{"GA"} accepts the following parameters:
-#' - "criterion" criterion to optimize. Default: BAR
-#' - "suggestions" suggestions to warm start the GA.
-#'   \code{NA} means no warm start. Default: TSP, QAP_LS and Spectral.
-#' - "selection" Selection operator.
-#' - "crossover" Crossover operator.
-#' - "mutation" Mutation operator. Default: a
-#'   mixture of the simple insertion (80% chance) and simple inversion (20%
-#'   chance) operators.
-#' - "pmutation" probability for permutations. Default: .5
-#' - "pcrossover" probability for crossover. Default: .2
-#' - "popsize" the population size. Default: 100
-#' - "maxiter" maximum number of generations. Default: 1000
-#' - "run" stop after \code{run} generations without improvement. Default: 50
-#' - "parallel" use multiple cores? Default: TRUE
-#' - "verbose" a logical; report progress? Default: TRUE
-#'
 #' The GA uses by default the ordered cross-over (OX) operator. For mutation,
 #' the GA uses a mixture of simple insertion and simple inversion operators.
 #' This mixed operator is created using
@@ -58,6 +36,8 @@
 #' Set \code{"suggestions"} to \code{NA} to start with a purely random initial
 #' population.
 #'
+#' See Example section for available control parameters.
+#'
 #' \bold{Note:} Package \pkg{GA} needs to be installed.
 #'
 #' @aliases register_GA GA ga gaperm_mixedMutation
@@ -75,38 +55,54 @@
 #' register_GA()
 #' get_seriation_method("dist", "GA")
 #'
-#' d <- dist(random.robinson(50, pre=TRUE, noise=.1))
+#' data(SupremeCourt)
+#' d <- as.dist(SupremeCourt)
 #'
-#' ## use default settings: Banded AR form
-#' o <- seriate(d, "GA")
+#' ## optimize for linear seriation criterion (LS)
+#' o <- seriate(d, "GA", criterion = "LS", verbose = TRUE)
 #' pimage(d, o)
 #'
-#' ## optimize for linear sertiation criterion (LS)
-#' o <- seriate(d, "GA", control = list(criterion = "LS"))
+#' ## Note that by default the algorithm is already seeded with a LS heuristic.
+#' ## This run is no warm start (no suggestions) and increase run to 100
+#' o <- seriate(d, "GA", criterion = "LS", suggestions = NA, run = 100,
+#'   verbose = TRUE)
 #' pimage(d, o)
 #'
-#' ## no warm start
-#' o <- seriate(d, "GA", control = list(criterion = "LS", suggestions = NA))
+#' o <- seriate(d, "GA", criterion = "LS", suggestions = NA, run = 100,
+#'   verbose = TRUE,  )
+#'
 #' pimage(d, o)
 #' }
 #' @export
 register_GA <- function() {
   check_installed("GA")
 
-  .ga_contr <- list(
+  .ga_contr <- structure(list(
     criterion = "BAR",
     suggestions = c("TSP", "QAP_LS", "Spectral"),
-    selection = GA::gaperm_nlrSelection,
+    selection = GA::gaperm_lrSelection,
     crossover = GA::gaperm_oxCrossover,
     mutation = gaperm_mixedMutation(.8),
-    pcrossover = .2,
-    pmutation = .5,
+    pcrossover = .8,
+    pmutation = .1,
     popSize = 100,
     maxiter = 1000,
     run = 50,
-    parallel = TRUE,
-    verbose = TRUE
-  )
+    parallel = FALSE,
+    verbose = FALSE
+  ), help = list(
+    criterion = "criterion to be optimized",
+    suggestions = "seed the population with these seriation methods",
+    selection = "selection operator function",
+    crossover = "crossover operator function",
+    mutation = "mutation operator function",
+    pcrossover = "probability for crossover",
+    pmutation = "ptobability of mutations",
+    popSize = "population size",
+    maxiter = "maximum number of generations",
+    run = "stop after run generations without improvement",
+    parallel = "use multiple cores?"
+  ))
 
   GA_helper <- function(x, control) {
     n <- attr(x, "Size")
@@ -114,7 +110,8 @@ register_GA <- function() {
     control <- .get_parameters(control, .ga_contr)
 
     if (control$verbose)
-      cat("\nPreparing suggestions\n")
+      cat("\nPreparing suggestions:",
+          paste0(control$suggestions, collapse = ", "), "\n")
 
     if (is.na(control$suggestions[1]))
       suggestions <- NULL
@@ -126,11 +123,10 @@ register_GA <- function() {
     if (control$verbose)
       cat("\nStarting GA\n")
 
-    ### FIXME: need to be able to set bandwidth for BAR
     # fitness function
     f <-
       function(o)
-        - criterion(x, o, method = control$criterion, force_loss = TRUE)
+        - criterion(x, as.integer(o), method = control$criterion, force_loss = TRUE)
 
     result <- GA::ga(
       type = "permutation",
@@ -144,10 +140,7 @@ register_GA <- function() {
       pcrossover = control$pcrossover,
       suggestions = suggestions,
       names = as.character(1:n),
-      monitor = if (control$verbose)
-        GA::gaMonitor
-      else
-        NULL,
+      monitor = control$verbose,
       parallel = control$parallel,
       maxiter = control$maxiter,
       run = control$run,
@@ -155,7 +148,12 @@ register_GA <- function() {
       popSize = control$popSize
     )
 
-    as.integer(result@solution[1,])
+    if (control$verbose)
+      if (result@iter < control$maxiter)
+        cat("\nStopped early after", control$run, "iterations with no improvement! (control option 'run')\n")
+
+    # solution may have multiple rows! Take the first solution.
+    as.integer(result@solution[1, , drop = TRUE])
   }
 
   set_seriation_method(
@@ -163,7 +161,10 @@ register_GA <- function() {
     "GA",
     GA_helper,
     "Use a genetic algorithm to optimize for various criteria.",
-    .ga_contr
+    .ga_contr,
+    randomized = TRUE,
+    optimizes = .opt(NA, "specified as parameter criterion"),
+    verbose = TRUE
   )
 }
 
